/* cdpd_bkgnoise.ps.c */                                                       
/* Custom background noise model for radio link Transceiver Pipeline */

/****************************************/
/*		Michael Jiang		    */
/****************************************/

#include <opnet.h>
#include <math.h>


/***** constants *****/

#define BOLTZMANN			1.379E-23
#define BKG_TEMP			290.0
#define AMB_NOISE_LEVEL		1.0E-26

#define KT					-174.0  /* dBm/Hz */

#define	RURAL				0
#define	SUBURBAN			1
#define	URBAN				2

/***** procedure *****/

void
cdpd_bkgnoise (pkptr)
	Packet*		pkptr;
	{
	double		rx_noisefig, rx_temp, rx_bw;
	double		bkg_temp, bkg_noise, amb_noise;
	double		ig_noise, em_noise;
	int			setting_type;

	/** Compute noise sources other than transmission interference. **/
	FIN (cdpd_bkgnoise (pkptr))

	/* Get receiver noise figure. */
	rx_noisefig = op_td_get_dbl (pkptr, OPC_TDA_RA_RX_NOISEFIG);

	/* Calculate effective receiver temperature. */
	rx_temp = (rx_noisefig - 1.0) * 290.0;

	/* Set the effective background temperature. */
	bkg_temp = BKG_TEMP;

	/* Get receiver channel bandwidth (in Hz). */
	rx_bw = op_td_get_dbl (pkptr, OPC_TDA_RA_RX_BW);

	/* Calculate in-band noise from both background and thermal sources. */
	bkg_noise = KT + (10 * (log10 (rx_bw)));

	/* Calculate in-band ambient noise. */
	/*amb_noise = rx_bw * AMB_NOISE_LEVEL;*/
	amb_noise = 0.0;

	/* Get the setting. First set the variable to the default value */
	/* and then overwrite it, if another value is explicitly set    */
	/* for the simulation attribute.                                */
	setting_type = URBAN;
	if (op_ima_sim_attr_exists ("setting_type") == OPC_TRUE)
	    {
	    op_ima_sim_attr_get (OPC_IMA_INTEGER, "setting_type", &setting_type);
	    }

	/* Initial em and ig noise settings. */
	ig_noise = 0.0;
	em_noise = 0.0;

	/* Add in ignition noise. */
	if (setting_type == RURAL)
		ig_noise = 1.0;

	else if (setting_type == SUBURBAN)
		ig_noise = 3.0;

	else if (setting_type == URBAN)
		ig_noise = 5.0;

	/* Add in 800 MHz Emission Noise (based on the analysis */
	/* done by "William C. Y. Lee" in the book "Mobile      */
	/* Cellular Telecommunications Systems").               */
	if (setting_type == RURAL)
		em_noise = 1.0;

	else if (setting_type == SUBURBAN)
		em_noise = 2.0;

	else if (setting_type == URBAN)
		em_noise = 3.0;

	/* Put the sum of both noise sources in the packet transmission data attr. */
	op_td_set_dbl (pkptr, OPC_TDA_RA_BKGNOISE, (amb_noise + bkg_noise + ig_noise + em_noise));


	FOUT
	}

